package com.sun.istack.internal;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.lang.ref.WeakReference;

public interface Pool<T>
{
    @NotNull
    public abstract T take();
    
    public abstract void recycle(@NotNull T p0);
    
    public abstract static class Impl<T> implements Pool<T>
    {
        private volatile WeakReference<ConcurrentLinkedQueue<T>> queue;
        
        @NotNull
        @Override
        public final T take() {
            T t = this.getQueue().poll();
            if (t == null) {
                return this.create();
            }
            return t;
        }
        
        @Override
        public final void recycle(T t) {
            this.getQueue().offer(t);
        }
        
        private ConcurrentLinkedQueue<T> getQueue() {
            WeakReference<ConcurrentLinkedQueue<T>> q = this.queue;
            if (q != null) {
                ConcurrentLinkedQueue<T> d = (ConcurrentLinkedQueue)q.get();
                if (d != null) {
                    return d;
                }
            }
            ConcurrentLinkedQueue<T> d = new ConcurrentLinkedQueue();
            this.queue = new WeakReference(d);
            return d;
        }
        
        @NotNull
        protected abstract T create();
    }
}
